--- $Id: Schema.sql,v 1.1.1.1 2003/01/23 05:29:37 sunday Exp $
--- Ogg Jukebox Schema
--- Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
--- Redistribution and use in source and binary forms, with or 
--- without modification, are permitted provided that the following 
--- conditions are met: 
--- 
--- - Redistributions of source code must retain the above copyright 
---   notice, this list of conditions and the following disclaimer. 
---
--- - Redistributions in binary form must reproduce the above copyright 
---   notice, this list of conditions and the following disclaimer in 
---   the documentation and/or other materials provided with the 
---   distribution. 
--- 
--- THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS 
--- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
--- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
--- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE 
--- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
--- OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
--- OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
--- OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
--- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
--- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
--- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
---

CREATE TABLE albums (
    id SERIAL PRIMARY KEY,
    title TEXT NOT NULL,
    artist TEXT NOT NULL DEFAULT 'Various'
);

CREATE TABLE tracks (
    album INTEGER NOT NULL REFERENCES albums( id ),
    track INTEGER NOT NULL,
    title TEXT NOT NULL,
    artist TEXT NOT NULL,
    length INTERVAL NOT NULL,
    PRIMARY KEY( album, track )
);

CREATE TABLE playlists (
    name TEXT PRIMARY KEY,
    last INTEGER NOT NULL
);

CREATE TABLE playlist_entries (
    name TEXT NOT NULL,
    position INTEGER NOT NULL,
    album INTEGER NOT NULL,
    track INTEGER NOT NULL,
    PRIMARY KEY( name, position ),
    FOREIGN KEY( name ) REFERENCES playlists( name ) 
      ON UPDATE CASCADE 
      ON DELETE CASCADE,
    FOREIGN KEY( album, track ) REFERENCES tracks( album, track )
      ON UPDATE CASCADE
      ON DELETE CASCADE
);

--- View Definitions

CREATE VIEW album_detail AS
    SELECT tracks.album, albums.artist, albums.title, 
           sum( tracks.length ) as length, sum( tracks.track ) as tracks
      WHERE tracks.album = albums.id
      GROUP BY tracks.album, albums.artist, albums.title
      ORDER BY albums.artist, albums.title;

CREATE VIEW track_detail AS
    SELECT tracks.album, tracks.track, albums.title AS album_title,
           tracks.title, tracks.artist, tracks.length
      WHERE albums.id = tracks.album
      ORDER BY albums.title, tracks.album, tracks.track;

CREATE VIEW playlist_detail AS
    SELECT p.name, p.last, count( t.track ) AS tracks, 
           sum( t.length ) AS length
      FROM playlists p, playlist_entries pe, tracks t
      WHERE p.name = pe.name
        AND pe.album = t.album 
        AND pe.track = t.track
      GROUP BY p.name, p.last
      ORDER BY p.name;

--- Permisions

GRANT SELECT, UPDATE, INSERT, DELETE ON albums TO media;
GRANT SELECT, UPDATE, INSERT, DELETE ON tracks TO media;
GRANT SELECT, UPDATE, INSERT, DELETE ON playlists TO media;
GRANT SELECT, UPDATE, INSERT, DELETE ON playlist_entries TO media;
GRANT SELECT ON album_detail TO media;
GRANT SELECT ON track_detail TO media;
GRANT SELECT ON playlist_detail TO media;

