/* $Id: ArgVector.C,v 1.1.1.1 2003/01/23 05:29:37 sunday Exp $
 * Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Description:
 *   Class to parse a string into tokens separated by spaces.
 *   Escaped and quoted strings are treated appropriately.
 *   Class may also be used as a cheap lookup table where
 *   input string is treated as a list of key/value pairs.
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "ArgVector.h"

ArgVector::ArgVector( const char *s ) :
  scratch( new char[strlen(s)+1] )
{
    int token = 0;
    const char *p = s;
    char *q = scratch;
    char *b = q;
    int quo = 0;
    
    while( *p ) {
        while( *p && *p == ' ' || *p == '\t' ) p++;
        if( !(*p) ) break;
        if( *p == '"' ) {
            quo = 1;
            p++;
        }
        b = q;
        token = 0;
        while( !token && *p) {
            if( *p == '\\' ) {
                p++;
            } else if( *p == '"' && quo ) {
                quo = 0;
                token = 1;
                *(q++) = 0;
                p++;
                break;
            } else if( *p == ' ' && !quo ) {
                token = 1;
                *(q++) = 0;
                p++;
                break;
            } 

            *(q++) = *(p++);
            *q = 0;
       }
       if( strlen( b ) > 0 ) {
           args.push_back( b );
       }
    }
}

ArgVector::~ArgVector() {
    delete [] scratch;
}

void ArgVector::Pop() {
    args.erase( args.begin() );
}

const char *ArgVector::HashFind( const char *key ) const {
    unsigned int i;
    for( i = 0; i < args.size(); i+=2 ) {
        if( strcasecmp( args[i], key ) == 0 ) {
            return (*this)[i+1];
        }
    }
    return NULL;
}
 
unsigned int ArgVector::size() const {
    return args.size();
}

const char *ArgVector::operator[](unsigned int index) const {
    if( index < size() ) {
        return args[index];
    }
    return NULL;
}


