/* $Id: ArgVector.h,v 1.3 2003/04/11 22:52:33 sunday Exp $
 * Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Description:
 *   Class to parse a string into tokens separated by spaces.
 *   Escaped and quoted strings are treated appropriately.
 *   Class may also be used as a cheap lookup table where
 *   input string is treated as a list of key/value pairs.
 */

#ifndef __ARGVECTOR_H__
#define __ARGVECTOR_H__
#include <vector>

using namespace std;

class ArgVector {
  public:
    ArgVector( const char *s );
    ~ArgVector();

    /* Pop
     * Remove the first item from the vector.
     */
    void Pop();

    /* HashFind
     * Treat the vector like a key map
     * Return the element after the argument that contains key, or
     * null if not found.
     */
    const char *HashFind( const char *key ) const;

    /* size
     * Return the # of elements in the vector
     */ 
    unsigned int size() const;

    /* operator[]
     * Return the indexth element.
     */
    const char *operator[](unsigned int index) const; 

  private:
    char *scratch;
    vector<char *> args;
};

#endif

