/* $Id: Database.h,v 1.4 2003/04/13 08:46:26 sunday Exp $
 * Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Description:
 *   Class to handle database connection and queries.
 *
 */

#ifndef __DATABASE_H__
#define __DATABASE_H__

#include <stdlib.h>
#include <string>
#include <libpq-fe.h>
#include <vector>

#include "ArgVector.h"
#include "Song.h"
#include "Album.h"

using namespace std;

class Database {
  public:
    Database();
    ~Database();

    public:
        Song *CreateSong( unsigned int album, unsigned int track );
        vector<Song*> *SearchTracks( const ArgVector &argv );
        Song *RandomTrack();
 
        vector<Album*> *SearchAlbums( const ArgVector &argv );

        vector<Song*> *LoadPlaylist( const char *name, unsigned int *index );
        int SavePlaylist( const char *name, vector<Song*> *list, 
            unsigned int index );
    private:
        vector<Song*> *TrackQuery( const char *query );
        static int WhereString( char **where, const char *column, 
          const char *value, int *prefix );
        static int WhereInt( char **where, const char *column,
          int value, int *prefix );

        PGconn *conn;
};

#endif
