/* $Id: Decode.h,v 1.3 2003/04/11 21:25:42 sunday Exp $
 * Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Description:
 *   Class to handle asynchronous OggVorbis decoding.
 *
 */

#ifndef __DECODE_H__
#define __DECODE_H__

#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>
#include <pthread.h>

class OggError {
  public:
    enum error { NO_FILE, NO_OGG };

    OggError( error e );
    const char *ToString() const;

  private:
    static char *text[];
    error err;
};

class OggFile {
  public:
    OggFile( int album, int track ) throw( OggError );
    ~OggFile();

    int Read( char *buf, unsigned int len );
    int GetAlbum() const;
    int GetTrack() const;

    double GetPosition() const;
    double GetLength() const;
    int GetSection() const;
    int GetChannels();
    int GetRate();

  private:
    int album;
    int track;
    int section;
    double length;
    double position;
    OggVorbis_File vf;
    int endian;
};

class OggDecoder {
  public:
    OggDecoder ();
    ~OggDecoder();   

    int RegisterNextCallback( void(*cb)(void *), void *arg );

    int ChangeTrack( int album, int track );
    int Pause();
    int Resume();

    #define DECODE_STOP  0x01
    #define DECODE_PLAY  0x02
    #define DECODE_PAUSE 0x03

    int Status( int *album, int *track, double *pos, double *len );

    void *Thread();

  private:
    #define ACTION_CONTINUE 0x00
    #define ACTION_NEXT     0x01
    #define ACTION_STOP     0x02

    OggFile *current;
    OggFile *next;

    void (*next_callback)(void*);
    void *next_callback_arg;
    int action;

    pthread_mutex_t next_mutex;
    pthread_mutex_t action_mutex;
    pthread_cond_t action_cond;
    pthread_t decode_thread;
};

#endif

