/* $Id: sound.c,v 1.3 2003/04/11 21:15:05 sunday Exp $
 * Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Description:
 *   Stub file for FreeBSD /dev/dsp output
 *
 */

#include <errno.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <ao/ao.h>

#include "sound.h"

static ao_device *sounddev;

static int byte_format;

int sound_init() {
    int driver_id;
    struct ao_sample_format sf;
    struct ao_info *ai;

    sf.bits = 16;
    sf.rate = 44100;
    sf.channels = 2;
    sf.byte_format = AO_FMT_BIG;

    ao_initialize();

    driver_id = ao_default_driver_id();
    if( driver_id < 0 ) {
        fprintf( stderr, "Unable to get default ao driver\n" );
        exit( 1 );
    }
    ai = ao_driver_info( driver_id ); 
    byte_format = ai->preferred_byte_format;

    sounddev = ao_open_live( driver_id, &sf, NULL );

    if( !sounddev ) {
        fprintf( stderr, "Unable to open ao device.\n" );
        exit( 1 );
    }

    return 0;
}

int sound_vorbis_endian() {
    return (byte_format == AO_FMT_LITTLE )?0:1;
}

int sound_setup( int channels, int rate ) {
    /* This is broken with ao. Fix later. */
    if( !sounddev ) {
        return -1;
    }
    return 0;
}

int sound_play( char *buf, int len ) {
    return ao_play( sounddev, buf, len );
}
