#!/usr/bin/perl

use DBI;
my $fileroot = "/music";

my $dbh = DBI->connect( 'dbi:Pg:dbname=ogg;host=localhost', 'media', '',
    {RaiseError => 1, AutoCommit => 0} ) || die "Couldn't connect to database";

my $album_sql = "SELECT artist, title FROM albums WHERE id = ?";
my $track_sql = "SELECT track, artist, title, length FROM tracks WHERE album = ? ORDER BY track";

my $sth = $dbh->prepare( $album_sql );
$sth->execute( $ARGV[0] ) or die $dbh->errstr;
my( $artist, $title, $track, $length );
$sth->bind_columns( undef, \$artist, \$title );
$sth->fetch();
die "No album found." if( !$title );

my $pak = "$artist -- $title.pak";
print "Creating \"$pak\"..\n";
open TARBALL, ">$pak" || die "Couldn't open output pak";
my $dir = sprintf "%s/%.8x", $fileroot, $ARGV[0];
chdir $dir || die "Couldn't chdir to $dir";
open XML, ">index.xml" || die "Couldn't open index file.";
print XML "<?xml version=\"1.0\"?>\n";
print XML "<!DOCTYPE Album SYSTEM \"jukealbum.dtd\">\n";
print XML "<Album>\n";
print XML "  <Artist>$artist</Artist>\n";
print XML "  <Title>$title</Title>\n";
$sth->finish();

$sth = $dbh->prepare( $track_sql );
$sth->execute( $ARGV[0] ) or die $dbh->errstr;
$sth->bind_columns( undef, \$track, \$artist, \$title, \$length );
while( $sth->fetch() ) {
	print XML "  <Track Index=\"$track\" Length=\"$length\">\n";
	print XML "    <Artist>$artist</Artist>\n";
	print XML "    <Title>$title</Title>\n";
	print XML "  </Track>\n";
}
print XML "</Album>\n";

close XML;
$dbh->disconnect();
if( fork() == 0 ) {
	open STDOUT, ">&TARBALL" || die "dup";
	exec "tar -cf - index.xml *.ogg";
}
wait();

