#!/usr/bin/perl

use DBI;

my $fileroot = '/music';
my $archive = $ARGV[0];

my $dbuser = 'media';
my $dbpass = '';
my $dbname = 'ogg';
my $dbhost = 'localhost';

mkdir "$fileroot/tmp";
system "gtar -xf \"$archive\" -C $fileroot/tmp";

open XML, "$fileroot/tmp/index.xml" || die "Couldn't open Index.xml";

my $dbh = DBI->connect( "dbi:Pg:dbname=$dbname;host=$dbhost", $dbuser, $dbpass, {RaiseError => 1, AutoCommit => 0} )
  or die $DBI::errstr;

my $sql = "INSERT INTO albums( title, artist ) VALUES( ?, ? )";
my $album_sth = $dbh->prepare( $sql );

$sql = qq{ 
    INSERT INTO tracks( album, track, title, artist, length ) 
      VALUES( ?, ?, ?, ?, ? )
};
my $track_sth = $dbh->prepare( $sql );

$sql = qq{ SELECT id FROM albums WHERE oid = ? };
my $id_sth = $dbh->prepare( $sql );

while( <XML> ) {
	if( /\<Album\>/ ) {
		while( <XML> ) {
			if( /\<Artist\>(.*)\<\/Artist\>/ ) {
				$artist = $1;
			}

			if( /\<Title\>(.*)\<\/Title\>/ ) {
				$title = $1;
			}

			if( /\<Track Index=\"(\d+)\" Length=\"([\d\:]+)\"\>/ ) {
				$track = {};
				$track->{index} = $1;
				$track->{length} = $2;
				while( <XML> ) {
					if( /\<Artist\>(.*)\<\/Artist\>/ ) {
						$track->{artist} = $1;
					}
					if( /\<Title\>(.*)\<\/Title\>/ ) {
						$track->{title} = $1;
					}
					last if( /\<\/Track\>/ );
				}
				push @tracks, $track;
			}
			
			last if( /\<\/Album\>/ );
		}
	}
}

print "Artist: $artist\n";
print "Title: $title\n";

$res = $album_sth->execute( $title, $artist );
$res = $id_sth->execute( $album_sth->{pg_oid_status} );
($album_id) = $id_sth->fetchrow_array();
$id_sth->finish;
printf ("Importing as album %.8x\n", $album_id );

for $t (@tracks) {
	$res = $track_sth->execute( $album_id, $t->{index}, $t->{title}, $t->{artist}, $t->{length} );
	print "$t->{index} $t->{title} -- $t->{artist} ($t->{length})\n";
}

$newdir = sprintf( "%s/%.8x", $fileroot, $album_id );
system "mv $fileroot/tmp $newdir";

$dbh->commit();
$dbh->disconnect();
